package net.hostsharing.hsadminng.rbac.rbacuser;

import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.Repository;

import java.util.List;
import java.util.UUID;

public interface RbacUserRepository extends Repository<RbacUserEntity, UUID> {

    @Query("""
             select u from RbacUserEntity u
                 where :userName is null or u.name like concat(cast(:userName as text), '%')
                 order by u.name
            """)
    List<RbacUserEntity> findByOptionalNameLike(String userName);

    // bypasses the restricted view, to be able to grant rights to arbitrary user
    @Query(value = "select * from rbacuser where name=:userName", nativeQuery = true)
    RbacUserEntity findByName(String userName);

    RbacUserEntity findByUuid(UUID uuid);

    @Query(value = "select * from grantedPermissions(:userUuid)", nativeQuery = true)
    List<RbacUserPermission> findPermissionsOfUserByUuid(UUID userUuid);

    /*
        Can't use save/saveAndFlush from SpringData because the uuid is not generated on the entity level,
        but explicitly, and then SpringData check's if it exists using an SQL SELECT.
        And SQL SELECT needs a currentUser which we don't yet have in the case of self registration.
     */
    @Modifying
    @Query(value = "insert into RBacUser_RV (uuid, name) values( :#{#newUser.uuid}, :#{#newUser.name})", nativeQuery = true)
    void insert(final RbacUserEntity newUser);

    default RbacUserEntity create(final RbacUserEntity rbacUserEntity) {
        if (rbacUserEntity.getUuid() == null) {
            rbacUserEntity.setUuid(UUID.randomUUID());
        }
        insert(rbacUserEntity);
        return rbacUserEntity;
    }

    void deleteByUuid(UUID userUuid);
}
